--[[
	Copyright (C) 2013 Mikhail Emelchenkov (http://mikhail.guru)
	Distributed under Microsoft Reference Source License (MS-RSL)
]]--

-- Hard-coded paths to run program remotely via Koneki Lua Remote Application
package.path = "./?.lua;/usr/local/share/lua/5.1/?.lua;/usr/local/share/lua/5.1/?/init.lua;/usr/local/lib/lua/5.1/?.lua;/usr/local/lib/lua/5.1/?/init.lua;/usr/share/lua/5.1/?.lua;/usr/share/lua/5.1/?/init.lua"
package.cpath = "./?.so;/usr/local/lib/lua/5.1/?.so;/usr/lib/lua/5.1/?.so;/usr/local/lib/lua/5.1/loadall.so;"
-- LZO module path
package.cpath = package.cpath .. "/root/lua_so/?.so;/root/lua_so/lua_?.so"

require "luarocks.loader"
nio = require "nixio"		-- Support for working with large files
require "struct"
md5 = require "md5"
require "pluto"		-- Proprietary module
require "lzo"		-- Proprietary module
require "utils"

local systemBinPath = "/root/new/system.bin"
local systemBinLzoPath = "/root/new/system.bin.lzo"
local systemBinMD5 = "/root/new/system.bin.md5"
local firmwarePath = "/root/original/firmware.bin"
local firmwareOutPath = "/root/new/firmware.bin"
local userImagePath = "/root/new/userImage.tar.gz"
local dataImagePath = "/root/new/dataImage.tar.gz"
local context = { data = "", dataPtr = 0 }
	
local function main()

	context.file = nio.open(firmwarePath, "r")
	context = readHeaders(context)
	
	local systemBin = nio.open(systemBinPath, "r")
	
	-- Check if system.bin.lzo and system.bin.md5 already exists
	local stat1, err1 = nio.fs.stat(systemBinLzoPath)
	local stat2, err2 = nio.fs.stat(systemBinMD5)
	if (err1 or err2) then
		print "system.bin.lzo and/or system.bin.md5 do not exist. Compress it and calculate MD5 sum."
		
		local inFile = nio.open(systemBinPath, "r")
		local outFile = nio.open(systemBinLzoPath, "w")
		print "Compressing..."
		compressSystemBin(inFile, outFile, context.upgradeFileHeader.sectionTable[2].flag.compression_size)
		print "Compress done"
		inFile:close()
		outFile:close()
		
		print "Calculating MD5..."
		local crc = calculateMD5(systemBinPath)
		print "Calculation done"
		outFile = nio.open(systemBinMD5, "w")
		outFile:write(crc)
		outFile:close()
	end
	
	-- Calculate new headers
	local inFile = nio.open(systemBinMD5, "r")
	local crc = inFile:read(16)
	inFile:close()
	local originalSize = nio.fs.stat(systemBinPath, "size")
	local compressedSize = nio.fs.stat(systemBinLzoPath, "size")
	local dataImageSize = nio.fs.stat(dataImagePath, "size")
	local userImageSize = nio.fs.stat(userImagePath, "size")
	local newHeader = newHeaders(context, crc, originalSize, compressedSize, dataImageSize, userImageSize)
	
	-- Build new firmware
	print "Writing new firmware"
	nio.fs.unlink(firmwareOutPath)
	newFirmware = nio.open(firmwareOutPath, "w")
	print "Writing header..."
	newFirmware:write(newHeader)
	print "Copying compressed kernel.bin..."
	context.file:seek(3872, "set")		-- 3872 bytes is an original header size (and new too)
	dd_buf(context.file, newFirmware, context.upgradeFileHeader.sectionTable[2].offset)
	newFirmware:close()
	
	print "Writing compressed system.bin..."
	local procHndl = io.popen(string.format("cat %s >> %s", systemBinLzoPath, firmwareOutPath))
	procHndl:read(); procHndl:close();
	print "Writing zero byte..."
	procHndl = io.popen(string.format("dd if=/dev/zero bs=1 count=1 >> %s", firmwareOutPath))
	procHndl:read(); procHndl:close();
	print "Writing dataImage.tar.gz..."
	procHndl = io.popen(string.format("cat %s >> %s", dataImagePath, firmwareOutPath))
	procHndl:read(); procHndl:close();
	print "Writing userImage.tar.gz..."
	procHndl = io.popen(string.format("cat %s >> %s", userImagePath, firmwareOutPath))
	procHndl:read(); procHndl:close();
	print "New firmware written"
	
	context.file:close()
end
main()

